/* array.h
 *
 * Copyright 2018 Bruce Cowan <bruce@bcowan.me.uk>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#pragma once

#include <stdbool.h>
#include <stddef.h>

typedef struct _Array Array;

typedef void   (*FreeFunc)    (void *element);
typedef double (*ValueFunc)   (const void *element);
typedef int    (*CompareFunc) (const void *a, const void *b);

// Fundamental functions
Array * array_new         (FreeFunc    func);
Array * array_ref         (Array      *array);
void    array_unref       (Array      *array);

void    array_add         (Array      *array,
                           const void *val);
bool    array_remove_fast (Array      *array,
                           size_t      index);
bool    array_remove      (Array      *array,
                           size_t      index);
bool    array_set         (Array      *array,
                           size_t      index,
                           const void *val);

// Getters    
const void * array_get          (const Array *array,
                                 size_t       index);
size_t       array_get_length   (const Array *array);
size_t       array_get_capacity (const Array *array);

// Misc
void array_sort (Array      *array,
                 CompareFunc func);
